function [K_A,CLW,gam,info,CL,K_A_red] = KAsyn_B3(G_A,SampleTime,varargin)
%KASYN synthesizes a H-inf controller for the Vireo.

%% Setup
d2r = pi/180;
if nargin == 3 && ischar(varargin{1})
    if strcmpi(varargin{1},'suppressplot')
        sp = false;
    else
        error(['I do not know the option "',varargin{1},'".']);
    end
elseif nargin == 2
    sp = true;
else
    error('Incorrect number/class of input arguments.');
end

%% ------ Aileron controller design ------
% Weights
W1 = ss(6*d2r);
set(W1,'InputName','d1(1)','OutputName','W1(1)');
% W1 is set equal to a small disturbance compared to the aileron saturation
% limits of +/-25 deg.

sum1 = ss(sumblk('%ail_cmd = %W1 + u',G_A.InputName,W1.OutputName));
% sum1 adds the contribution of W1 to the plant input.

W2 = ss(diag([5*d2r,50*d2r]));
set(W2,'InputName',{'d2(1)','d2(2)'},'OutputName',{'W2(1)','W2(2)'});
% W2 is set equal to small noise values.

sum2 = ss(sumblk('yn = %y + %W2',G_A.OutputName,W2.OutputName));
% sum2 adds the contribution of W2 at the plant output.

W3 = ss(150*d2r);
set(W3,'InputName','d3(1)','OutputName','r(1)');

Wu = zpk([-3778 -38.73 -8.762 -0.897],[-2857 -872.2 -28.08 -1.887],60.31);
set(Wu,'InputName','u','OutputName','e1(1)');
% The gain increases at higher frequencies to penalize high frequency
% control commands.

Wy = ss(1/(250*d2r));
set(Wy,'InputName',G_A.OutputName(2),'OutputName','e2(1)');
% Wy helps in setting the damping required on the roll rate. phi is not
% weighted here because it gets weighted using Pphi (see below).

sum3 = ss(sumblk('ephi = %r - %phi_n',W3.OutputName,sum2.OutputName(1)));
% sum3 subtracts phi from phi_cmd to generate the tracking error.

int = ss(tf(1,[1,0]));
set(int,'InputName',sum3.OutputName,'OutputName',['I_',sum3.OutputName{:}]);
% int integrates the tracking error.

P11 = 0.2*2;
zcw = 4;
P22 = zcw*P11;
Pphi_sys = ss(zpk(-zcw,0,P11));
Pphi = ss([P11,P22]);
set(Pphi,'InputName',[sum3.OutputName,int.OutputName],'OutputName',{'e3'});
% Pphi is set by placing penalties on the tracking error and its integral.
% Since the tracking error and its integral (effectively low freq part) are
% separated, there is no need to make Pphi a dynamic weight.

% Generalized plant
gen_G_A = connect(G_A,W1,sum1,W2,sum2,W3,Wu,Wy,sum3,int,Pphi,...
    {'d1','d2','d3','u'},{'e1','e2','e3','ephi','I_ephi','yn(2)'});

%% Synthesis
nmeas = 3; % ['ephi','I_ephi','yn(2)']
ncon = size(Wu,2); % 'u'
[~,~,gam,~] = hinfsyn(ssbal(gen_G_A),nmeas,ncon);
[K_A_temp,CLW,gam,info] = hinfsyn(ssbal(gen_G_A),nmeas,ncon,...
    'GMIN',gam*1.05,'GMAX',gam*1.05); % backoff by 5% to avoid fast poles

%% Complete controller by connecting integrator and sum block
set(K_A_temp,'InputName',{'ephi','I_ephi','yn(2)'},'OutputName','u');
K_A = connect(K_A_temp,int,sum3,{'r','yn'},K_A_temp.OutputName);

%% Set controller properties
K_A.InputName = ['phi_cmd',G_A.OutputName'];
K_A.OutputName = G_A.InputName;
K_A.InputGroup = struct('Reference',1,'Measurement',[2,3]);
K_A.Name = 'Aileron controller';
K_A.Notes = {'Simulink units'};

%% Form closed-loop
CL = connect(G_A,K_A,{'phi_cmd'},[G_A.OutputName',G_A.InputName']);

%% Form unweighted closed-loop
CLUW = connect(G_A,K_A_temp,sum1,sum2,sum3,int,...
    {'W1','W2','r'},{'u','p','ephi','I_ephi'});

%% Analysis plots for tuning
if sp
    % Extract weighted transfer function matrices
    CLW11 = CLW('e1','d1'); % 1x1
    CLW12 = CLW('e1','d2'); % 1x2
    CLW13 = CLW('e1','d3'); % 1x1
    
    CLW21 = CLW('e2','d1'); % 1x1
    CLW22 = CLW('e2','d2'); % 1x2
    CLW23 = CLW('e2','d3'); % 1x1
    
    CLW31 = CLW('e3','d1'); % 2x1
    CLW32 = CLW('e3','d2'); % 2x2
    CLW33 = CLW('e3','d3'); % 2x1
    
    % Set parameters
    if ~exist('cnt','var')
        cnt = 1;
        figure('Name','Weighted CL for aileron control design');
        ls = {'c--','g--','b--','m--','r--','k--'};
        wmin = 0.1; wmax = 100; % [rad/s]
        ax = cell(1,9);
        for m = 1:9
            ax{m} = subplot(330+m);
            hold on;
        end
    else
        cnt = cnt + 1;
    end
    
    % Plot frequency responses
    % subplot(331); hold on;
    bodemag(ax{1},CLW11,ls{cnt}); title('CLW11'); xlim([wmin,wmax]);
    
    % subplot(332); hold on;
    sigmaplot(ax{2},CLW12,ls{cnt}); title('CLW12'); xlim([wmin,wmax]);
    
    % subplot(333); hold on;
    bodemag(ax{3},CLW13,ls{cnt}); title('CLW13'); xlim([wmin,wmax]);
    
    % subplot(334); hold on;
    bodemag(ax{4},CLW21,ls{cnt}); title('CLW21'); xlim([wmin,wmax]);
    
    % subplot(335); hold on;
    sigmaplot(ax{5},CLW22,ls{cnt}); title('CLW22'); xlim([wmin,wmax]);
    
    % subplot(336); hold on;
    bodemag(ax{6},CLW23,ls{cnt}); title('CLW23'); xlim([wmin,wmax]);
    
    % subplot(337); hold on;
    sigmaplot(ax{7},CLW31,ls{cnt}); title('CLW31'); xlim([wmin,wmax]);
    
    % subplot(338); hold on;
    sigmaplot(ax{8},CLW32,ls{cnt}); title('CLW32'); xlim([wmin,wmax]);
    
    % subplot(339); hold on;
    sigmaplot(ax{9},CLW33,ls{cnt}); title('CLW33'); xlim([wmin,wmax]);
    
    % Plot step response of closed-loop
    figure('Name','Step response of closed-loop'); step(CL,ls{cnt}); hold all;
    
    % Plot sensitivity of unweighted closed-loop
    T = CL('phi','phi_cmd'); S = 1 - T;
    Sbnd = 1/Pphi_sys; % bound on S
    figure('Name','Sensitivity of unweighted closed-loop: phi cmd to error');
    bodeplot(S,ls{cnt}); hold on; bodeplot(Sbnd,'k--'); xlim([wmin,wmax]);
end

%% Controller order reduction
if sp; figure('Name','HSV plot of controller'); hsvplot(K_A); end
% From the HSVPLOT it is seen that the last few states may be residualized.

[K_A_bal,g] = balreal(K_A); % balanced realization of K_A
n = order(K_A_bal); % order of controller synthesized
nr = 7; % number of states to residualize
indResid = (n-nr+1):n; % indices of states to be residualized
K_A_red = modred(K_A_bal,indResid); % balanced residualization
set(K_A_red,'Name',K_A.Name,'Notes',K_A.Notes);

%% Discretization
% K_A_DT = c2d(K_A_red,SampleTime); % convert to DT using C2D
% K_A_DT = c2d_pow(K_A_red,SampleTime,3); % convert to DT using C2D_POW
% set(K_A_DT,'Name',K_A.Name,'Notes',K_A.Notes);
end
