function [L,mmi,mmo,mmio] = Vireo_robana(G,K,cMode,SampleTime,varargin)
%VIREO_ROBANA is used to conduct a MIMO analysis of the closed-loop
%and extract information on the different CL sensitivity functions.

%% Setup
nn = 4;
if nargin == (nn+1) && ischar(varargin{1})
    if strcmpi(varargin{1},'suppressplot')
        sp = false;
    else
        error(['I do not know the option "',varargin{1},'".']);
    end
elseif nargin == nn
    sp = true;
else
    error('Incorrect number/class of input arguments.');
end

%% Set gain for measurements sent to controller
% Note that LOOPSENS and LOOPMARGIN both assume that the controller is in
% negative feedback with the plant. If this is true use +1; else use -1.
switch cMode
    case {'LQR','LQI'}
        % Resetting the C matrix as identity matrix (state feedback requirement)
        set(G,'c',eye(length(G.a)),'d',[],'OutputName',G.StateName);
        fprintf('\n - The C matrix has been reset as the identity for state feedback.\n');
        gn = +1; % LQR regulator is in negative feedback
    otherwise % default
        gn = -1; % generic controller is assumed to be in positive feedback
end
C = gn*K(:,'Measurement'); % only select 'Measurement' so that C is 1-DOF

%% Evaluate stability of controller in continuous and discrete time
if C.Ts == 0
    % Continuous time controller
    if sp
        fprintf(['\n------',cMode,': the controller poles (continuous time) are:------\n']);
        pole(C)
        if sum(real(pole(C))>0)>0
            warning('The continuous time controller has unstable poles.');
        end
    end
    CDT = c2d(C,SampleTime); % convert to discrete-time
else
    % Discrete time controller
    fprintf('\n - The controller is in discrete-time.\n');
    fprintf(' - Hence the plant is also being converted to discrete-time.\n');
    G = c2d(G,C.Ts); % plant must also be DT for LOOPSENS and LOOPMARGIN to work
    CDT = C; % C is already in discrete-time
end

% Discrete time controller
if sp
    fprintf(['\n------',cMode,': the controller poles (discrete time) are:------\n']);
    pole(CDT)
    if sum(abs(pole(CDT))>1)>0
        warning('The discrete time controller has unstable poles.');
    end
end

%% Compute the sensitivity functions of the closed-loop
% LOOPSENS considers the controller in negative feedback
L = loopsens(G,C);
if sp
    fprintf(['\n------',cMode,': the poles of the closed-loop are:------\n']);
    L.Poles
    if G.Ts == 0 && C.Ts == 0 % continuous-time closed-loop
        if sum(real(L.Poles)>0)>0
            warning('The closed-loop has unstable poles.');
        end
    elseif G.Ts ~= 0 && C.Ts ~=0 % discrete-time closed-loop
        if sum(abs(L.Poles)>1)>0
            warning('The closed-loop has unstable poles.');
        end
    else
        warning(['Between the plant and the controller, one of them is in ',...
            'continuous-time and the other is in discrete-time.']);
    end
end

%% Singular value plots of PSi, CSo, Si, So, and C
if sp
    % Transmission of disturbances at the plant input to the plant output: PSi
    figure('Name','Sigma plot of plant * Si'); sigmaplot(L.PSi);
    [gpeakPSi,fpeakPSi] = norm(L.PSi,inf);
    
    % Transmission of disturbances at the plant output to the plant input: CSo
    figure('Name','Sigma plot of controller * So'); sigmaplot(L.CSo);
    [gpeakCSo,fpeakCSo] = norm(L.CSo,inf);
    
    % Input sensitivity function: disturbances at plant input to control effort: Si
    figure('Name','Input-to-plant sensitivity function'); sigmaplot(L.Si);
    [gpeakSi,fpeakSi] = norm(L.Si,inf);
    
    % Output sensitivity function: disturbances at plant output to control error: So
    figure('Name','Output-to-plant sensitivity function'); sigmaplot(L.So);
    [gpeakSo,fpeakSo] = norm(L.So,inf);
    
    % Input-to-plant loop transfer function: Li
    figure('Name','Input-to-plant loop transfer function'); sigmaplot(L.Li);
    
    % Output-to-plant loop transfer function: Lo
    figure('Name','Output-to-plant loop transfer function'); sigmaplot(L.Lo);
    
    % Singular value plot of the controller
    figure('Name','Controller'); sigmaplot(C);
end

%% Print the peak frequency response of MIMO system
if sp
    fprintf(['\n------',cMode,': peak frequency response of MIMO system:------\n',...
        ' - ||PSi||_{inf} = %4.2f dB.\n',...
        ' - ||CSo||_{inf} = %4.2f dB.\n',...
        ' - ||Si||_{inf} = %4.2f dB.\n',...
        ' - ||So||_{inf} = %4.2f dB.\n'],...
        20*log10(gpeakPSi), 20*log10(gpeakCSo), 20*log10(gpeakSi), 20*log10(gpeakSo));
end

%% Compute the MIMO stability margins of the feedback loop
% LOOPMARGIN considers the controller in negative feedback
[cmi,dmi,mmi,cmo,dmo,mmo,mmio] = loopmargin(G,C);
if sp
    fprintf(['\n------',cMode,...
        ': multiloop channel margins at plant input:------\n']); mmi
    fprintf(['\n------',cMode,...
        ': multiloop channel margins at plant output:------\n']); mmo
    fprintf(['\n------',cMode,...
        ': multiloop channel margins at input and output:------\n']); mmio
end
end
